STANDARD_VAR_NAMES_NHTSA = {
    "ID": "Id",
    "GCWR": "GCWRFrom",
    "GCWR_to": "GCWRTo",
    "GVWR": "GVWRFrom",
    "GVWR_to": "GVWRTo",
    "Make_ID": "MakeId",
    "MakeID": "MakeId",
    "Make_Name": "Make",
    "MakeName": "Make",
    "Mfr_CommonName": "ManufacturerCommonName",
    "Mfr_ID": "ManufacturerId",
    "Mfr_Name": "Manufacturer",
    "ManufacturerName": "Manufacturer",
    "MfrId": "ManufacturerId",
    "MfrName": "Manufacturer",
    "Model_ID": "ModelId",
    "Model_Name": "Model",
    "ModelName": "Model",
    "ModelID": "ModelId",
    "VehicleTypeName": "VehicleType",
}